# Generated by Django 5.2.7 on 2025-10-22 20:57

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("authentication", "0007_alter_organizationcategory_options"),
    ]

    operations = [
        migrations.AlterField(
            model_name="platformorganization",
            name="platform_users",
            field=models.ManyToManyField(
                default=None,
                through="authentication.OrganizationMember",
                through_fields=["platform_organization", "platform_user"],
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="platformuser",
            name="organizations",
            field=models.ManyToManyField(
                default=None,
                related_name="members",
                through="authentication.OrganizationMember",
                through_fields=["platform_user", "platform_organization"],
                to="authentication.platformorganization",
            ),
        ),
    ]
