# Generated by Django 4.2.18 on 2025-01-17 05:49

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("authentication", "0003_platformuser_organizations_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="platformuser",
            name="user_image",
            field=models.ImageField(blank=True, null=True, upload_to=None),
        ),
        migrations.AlterField(
            model_name="organizationmember",
            name="platform_user",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="authentication.platformuser",
            ),
        ),
        migrations.AlterField(
            model_name="platformorganization",
            name="platform_users",
            field=models.ManyToManyField(
                default=None,
                through="authentication.OrganizationMember",
                to="authentication.platformuser",
            ),
        ),
    ]
